#!/usr/bin/perl
#=============================================================================
# FOAF Configure 
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
# Copyright (C) 2003           Masahiro Hamasaki, All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

use strict;

use lib qw(lib extlib);
use RNA::PlatformDep;
use CGI;
use CGI::Carp;
use Mail::SpamAssassin::SHA1;
use HTTP::Lite;
use RNA;
use RNA::Utils qw(:all);
use RNA::FOAF;

my $cgi = new CGI;
#----------------------------------
# 基本設定
#----------------------------------
my $self_uri = $cgi->url(-relative=>1);
my $self_uri_mode_edit = qq($self_uri?mode=edit);
my $self_uri_with_query = $cgi->self_url;

#==============================================================================
# main
#==============================================================================

# RNA オブジェクトの生成
my $rna = RNA->new(Config => './config.pl');
my $config = $rna->{Config};

my $mode = $cgi->param('mode');

#----------------------------------
# 認証
#----------------------------------
if($mode) {
	if ($mode !~ /^(hello|show_foaf|visualize)$/) {
	# mode = hello の時以外
		if($config->{Authentication}->{FOAFCGI}) {
			unless($rna->authenticate(cgi=>$cgi)){
				print $cgi->redirect($config->{AuthenticationCGIURI} 
							. '?transfer=' . url_encode($self_uri_with_query));
				exit;
			}
		}
	}
}

# CGI Parameters
my @params = $cgi->param;
my %cgi_param = ();
foreach my $param (@params) {
	my $value = $cgi->param($param);
	if ($value && $param ne 'mode') { 
		convert_char_code(\$value);
		$cgi_param{$param} = encode($value);
	}
}

if ($mode eq 'edit') {
	&show_main_page;
}
elsif ($mode eq 'update') {
# FOAF ファイル作成  
	&generate_foaf_file(\%cgi_param);
	print $cgi->redirect($self_uri_mode_edit);
}
elsif ($mode eq "edit_friend") {
	&edit_friend(\%cgi_param);
	print $cgi->redirect($self_uri_mode_edit);
}
elsif ($mode eq "add_friend") {
# add the new person to <knows>
	eval{&add_friend(\%cgi_param); };
	&die_handler($@) if($@);

	print $cgi->redirect($self_uri_mode_edit);

}
elsif ($mode eq 'clear_known') {
	&clear_known_list;
	print $cgi->redirect($self_uri_mode_edit);
}
elsif ($mode eq "hello") { 
# receive hello message
	&post_receive_hello(\%cgi_param); 
} 
elsif ($mode eq "visualize") {
# FOAF 情報表示（XSLT や Template 変換など）
	print $cgi->redirect($config->{FOAF}->{File});
}
elsif ($mode eq "show_foaf") {
# FOAF ファイルの内容表示
  my $str = read_file($config->{FOAF}->{File});
  print $cgi->header(-type=>'text/xml', -charset => $config->{Encoding});
  print $str;
}
else {
# default
	print $cgi->redirect($config->{FOAF}->{File});
}



##=============================================================================
## sub routine
##=============================================================================
#
#
#-----------------------------------------------------------------------------
# FOAF ファイル作成
#-----------------------------------------------------------------------------

##
## FOAFファイルの作成
##
sub generate_foaf_file {
	my $hash_ref = shift;
	my %hash = %{$hash_ref};

	#---------------------------
	#validation
	#---------------------------
	if(!$hash{mbox} || $hash{mbox} eq 'mailto:') {
		&die_handler("mbox is required. ") ;
	}
	if( !($hash{mbox} =~ /^[0-9a-fA-f]+$/) 
		&& substr($hash{mbox}, 0, 7) ne 'mailto:' ) 
	{
		&die_handler("mbox needs prefix 'mailto:'.");
	}

	my $encoding = $config->{Encoding};

	my $foaf = new RNA::FOAF(encoding=>$encoding);
	eval{$foaf->parsefile($config->{FOAF}->{File})};
	my @knows = @{$foaf->person->knows};
	$foaf->person(
		name       => $hash{name},
		title      => $hash{title},
		firstName  => $hash{firstName},
		surname    => $hash{surname},
		nick       => $hash{nick},
		homepage   => $hash{homepage},
		weblog     => $hash{weblog},
		depiction  => $hash{depiction},
		phone      => $hash{phone},
		rdfs       => {
			seeAlso => $config->{FOAF}->{URI}
		},
		semblog => {
			rna => $config->{BaseURI},
			foaftb => $config->{FOAFCGIURI}
		}
	);

	# mail address
	if( !($hash{mbox} =~ /[:@\.]/) ) {
	# not mail-address format.  --->  <foaf:mbox_sha1sum>
		$foaf->person->add_item(
			element=>'mbox_sha1sum', value=>$hash{mbox} );
	}
	elsif($hash{'mbox_encrypt'}) {
	# <foaf:mbox_sha1sum>
		$hash{'mbox_sha1sum'}  = &sha1($hash{mbox});
		delete $hash{mbox};

		$foaf->person->add_item(
			element=>'mbox_sha1sum', value=>$hash{'mbox_sha1sum'} );
	}
	else {
	# <foaf:mbox>
		$foaf->person->add_item(element=>'mbox', value=>$hash{mbox});
	}

	$foaf->person->{knows} = \@knows;

	&write_my_foaf($foaf);
}


#-----------------------------------------------------------------------------
# Friends の編集
#-----------------------------------------------------------------------------
# &edit_friend(\%cgi_param);
sub edit_friend
{
	my $hash_ref = shift;
	my %hash = %{$hash_ref};
	my $target_mbox = $hash{target_mbox};
	my $action = $hash{action};

	# my FOAF
	my $foaf = &get_my_foaf;

	if($action =~ /Delete/i) {
	# delete
		foreach my $i (0 .. scalar(@{$foaf->person->knows})-1 ) {
			my $p = $foaf->person->knows->[$i];
			if( $target_mbox eq ($p->{mbox}||$p->{'mbox_sha1sum'}) ) {
#$rna->log(level=>'debug', message=>"target $target_mbox, mbox:" . $p->{mbox} . ", mbox_sha1sum:" . $p->{'mbox_sha1sum'});
				splice(@{$foaf->person->knows}, $i, 1);
				last;
			}
		}
	}
	else {
		&die_handler("Invalid Action : $action");
	}

	# write
	&write_my_foaf($foaf);
}

#-----------------------------------------------------------------------------
# main ページの表示
#-----------------------------------------------------------------------------

# &show_main_page;
sub show_main_page
{
	print $cgi->header(-type=>'text/html', -charset => $config->{Encoding});

	# my FOAF
	my $foaf = &get_my_foaf;
	my $person = $foaf->person;
	my @knows = @{$foaf->person->knows};

	# <select>
	my $select = "<select name=\"target_mbox\">\n";
	$select .= "<option value=\"\"> -- friends -- </option>\n";
	foreach my $p (@knows) {
		my $name = encode($p->nick||$p->name);
		my $mbox = encode($p->mbox||$p->mbox_sha1sum);
		my $friend_foaf_uri = $p->seeAlso;

		$select .= "<option value=\"$mbox\">$friend_foaf_uri &nbsp\;&nbsp\;- $name [$mbox]</option>\n";
	}
	$select .= "</select>\n";

	# template
	my $str = read_file($config->{Template}->{FOAFConfigure});

	my %text_hash = %{$config->{Template}->{Element}};
	$text_hash{__RNAFOAFFriendSelect__} = $select;
	$text_hash{RNAFOAFPersonTitle}      = $person->{title};
	$text_hash{RNAFOAFPersonName}       = $person->{name};
	$text_hash{RNAFOAFPersonFirstName}  = $person->{firstName};
	$text_hash{RNAFOAFPersonSurname}    = $person->{surname};
	$text_hash{RNAFOAFPersonNick}       = $person->{nick};
	$text_hash{RNAFOAFPersonMbox} 
		= $person->{mbox} || $person->{'mbox_sha1sum'} || 'mailto:';
	$text_hash{RNAFOAFPersonMboxEncrypt}
		= ($person->{mbox} ? '' : qq( checked="checked"));
	$text_hash{RNAFOAFPersonWeblog}   = $person->{weblog} || '';
	$text_hash{RNAFOAFPersonHomepage}   = $person->{homepage} || '';
	$text_hash{RNAFOAFPersonDepiction}  = $person->{depiction} || '';
	$text_hash{RNAFOAFPersonPhone}      = $person->{phone} || 'tel:';
	foreach my $t (keys %text_hash){
		$str =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	$str =~ s/(\<RNAHelloRequests\>.*?\<\/RNAHelloRequests\>)/&generate_hello_list($1)/ges;

	print $str;
}

####
#### generate recently hello message list
####
sub generate_hello_list 
{
	my $template = shift;

	# <RNACategories> の削除
	$template =~ s/\<\/?RNAHelloRequests\>//gs;

	my $known_file = $config->{FOAF}->{KNOWN};
	unless(-f $known_file) {
		return '';
	}
	open(IN, $config->{FOAF}->{KNOWN}) 
		|| die_handler($config->{FOAF}->{KNOWN} . ": $!");

	my $ret_str = '';
	while (<IN>) {
		if (/^\#/) { next; }
		chomp; chomp;
		
		my $t = $template;
		my @data = split('\t', $_);
		my %text_hash = (
			RNAHelloRequestDate       => encode($data[0]),
			RNAHelloRequestSender     => encode($data[1]),
			RNAHelloRequestFOAFURI    => encode($data[2]),
			RNAHelloRequestFOAFCGIURI => encode($data[3])
		);
		foreach my $s (keys %text_hash){
			$t =~ s/\<\$$s\$\>/$text_hash{$s}/ges;
		}

		$ret_str .= $t;
	}
	close(IN);
	
	return $ret_str;
}



#-----------------------------------------------------------------------------
# <knows> リストへの追加
#-----------------------------------------------------------------------------

####
#### Add friend of other FOAF or send hello message
####
sub add_friend {
	my $hash_ref = shift;
	my %hash = %{$hash_ref};
	my $uri = $hash{'uri'};
	my $send_hello = $hash{'send_hello'};

	my $http = new HTTP::Lite;
	my $foaf_string = '';

	# get target FOAF
	my $foaf_uri = $hash{foaf_uri};
	my $foaf_get_uri = undef;
	if($foaf_uri) {
		$foaf_get_uri = $foaf_uri;
	}
	elsif($uri =~ /\.(xml|rss|rdf)$/) {
		$foaf_uri = $uri;
		$foaf_get_uri = $uri;
	}
	else {
		$foaf_get_uri = $uri . (($uri =~ /\?/) ? '&' : '?') . 'mode=show_foaf';
	}
	
	if ($http->request($foaf_get_uri) == 200) {$foaf_string = $http->body; }
	else { die "Request failed: " . $http->status_message();}

	my $trackback_uri = $hash{trackback_uri};
	if($send_hello) {
	# Send hello to FOAF Trackback CGI
		my $target_foaf = new RNA::FOAF;
		$target_foaf->parse($foaf_string);

		$trackback_uri ||= ($target_foaf->person->trackback_uri ? 
			$target_foaf->person->trackback_uri : $uri);

		&hello($trackback_uri) or die "Can't access $trackback_uri";
	}

	## add a knows_person
	my $my_foaf = &add_knows_person($foaf_string, $foaf_uri, $trackback_uri);
}

###
### knows personを追加
###
sub add_knows_person {
	my ($foaf_xml, $foaf_uri, $trackback_uri) = @_;
	
	## 相手のFOAFデータを読み込む
	my $target_foaf = new RNA::FOAF;
	eval {$target_foaf->parse($foaf_xml);};
	if($@) {
		die "$foaf_uri : Invalid FOAF. ($@)\n";
	}

	# <rdfs:seeAlso>
	unless($target_foaf->person->seeAlso) {
		if($foaf_uri) {
			$target_foaf->{person}->{rdfs}->{seeAlso} = $foaf_uri;
		}
	}

	# <semblog:foaftb>
	unless($target_foaf->person->foaftb) {
		if($trackback_uri) {
			$target_foaf->{person}->{semblog}->{foaftb} = $trackback_uri;
		}
	}

	$target_foaf->person->clear_knows;

	## 自分のFOAFデータを読み込む
	my $my_foaf = &get_my_foaf;
	# 現在の Friend リストを作成
	my %exist_mbox = ();
	foreach my $p (@{$my_foaf->person->knows}) {
		my $key = $p->{mbox} || $p->{'mbox_sha1sum'};
		$exist_mbox{$key} = 1;
	}

	# 追加
	my $key = $target_foaf->person->{mbox} 
		|| $target_foaf->person->{mbox_sha1sum};
	if($exist_mbox{$key}){
		&die_handler("mbox [$key] is already in the FOAF.");
	}
	else {
		push(@{$my_foaf->{person}->{knows}}, $target_foaf->{person});
	}

	## XML出力
	&write_my_foaf($my_foaf);

	return $my_foaf;
}


###
### hello communication
###
sub hello {
	my ($url) = @_;
	my $http = new HTTP::Lite;
	
	## make a POST request
	my %vars = (
		"mode"    => "hello",
		"name"    => $config->{Owner},
		"foaf_uri"    => $config->{FOAF}->{URI},
		"trackback_uri"   => $config->{FOAFCGIURI},
	);
	$http->prepare_post(\%vars);
	
	## HTTP communication
	my $request = $http->request($url);
	
	if ($request == 200) { return $http->body; }
	else { return undef; }
}


#-----------------------------------------------------------------------------
# KNOWN ファイルの処理
#-----------------------------------------------------------------------------

sub clear_known_list
{
	my $file = $config->{FOAF}->{KNOWN};
	my $str = '';
	print_file($file, \$str);
}


#-----------------------------------------------------------------------------
# Hello の受信
#-----------------------------------------------------------------------------
####
#### receive hello message
####
sub post_receive_hello {
  my $hash_ref = shift;
  my %hash = %{$hash_ref};
  my $trackback_uri = $hash{'trackback_uri'};
  my $foaf_uri      = $hash{'foaf_uri'};
  my $name          = $hash{'name'};
  
  ## output my foaf data
  my $str = read_file($config->{FOAF}->{File});
  print $cgi->header(-type=>'text/xml', -charset => $config->{Encoding});
  print $str;
  
  ## register data about the sender
  my $request = localtime() . "\t$name\t$foaf_uri\t$trackback_uri\n";
  convert_char_code(\$request);
  open(OUT, '>>' . $config->{FOAF}->{KNOWN}) || die_handler($!);
  print OUT $request;
  close(OUT);
}


# $foaf = &get_my_foaf;
sub get_my_foaf
{
	my $foaf = new RNA::FOAF;
	eval {$foaf->parsefile($config->{FOAF}->{File});};
	if($@) {
		&die_handler("Invalid FOAF file : $config->{FOAF}->{File}\n");
		return undef;
	}
	else {
		return $foaf;
	}
}

# &write_my_foaf($foaf);
sub write_my_foaf
{
	my $foaf = shift;

	my $str = $foaf->as_string;
	convert_char_code(\$str);
	print_file($config->{FOAF}->{File}, \$str);
}



# $str = encode($str)
# $str = encode($str);
sub encode
{
	my ($t) = @_;
	my %cv = ('"' => '&quot;', '<' => '&lt;',  '>' => '&gt;');
	$t =~ s/([<>"])/$cv{$1}||$1/ge;
	$t;
}
