#=============================================================================
# Copyright (C) 2001-2002 Sherzod Ruzmetov <sherzodr@cpan.org>. All rights reserved.
#
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#=============================================================================
# This file is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#=============================================================================
# Session driver for CGI::Session.
#
# $Id: FilePP.pm,v 1.1 2004/01/04 09:46:17 msano Exp $
#=============================================================================
package CGI::Session::FilePP;


use strict;
use File::Spec;
use Fcntl (':DEFAULT', ':flock');
use CGI::Session;
use Exporter;
use vars qw($VERSION @ISA $FileName);

@ISA = qw(Exporter CGI::Session CGI::Session::ID::MD5PP 
	CGI::Session::Serialize::FreezeThaw);
# @ISA = qw(Exporter CGI::Session CGI::Session::ID::MD5 CGI::Session::Serialize::Default);

eval {
	require Digest::MD5;
	import Digest::MD5 'md5_hex'
};
if ($@) { 
	require Digest::Perl::MD5;
	import Digest::Perl::MD5 'md5_hex'
}


my $build_date = join('', ('$Date: 2004/01/04 09:46:17 $' =~ /\d{2}(\d{2})\/(\d{2})\/(\d{2})/));
$VERSION = $build_date;

$FileName = 'cgisess_%s';

sub store {
	my ($self, $sid, $options, $data) = @_;

	$self->File_init($sid, $options);
	unless ( sysopen (FH, $self->{_file_path}, O_RDWR|O_CREAT|O_TRUNC, 0644) ) {
		$self->error("Couldn't store $sid into $self->{_file_path}: $!");
		return undef;
	}
	unless (flock(FH, LOCK_EX) ) {
		$self->error("Couldn't get LOCK_EX: $!");
		return undef;
	}
	print FH $self->freeze($data);	
	unless ( close(FH) ) {
		$self->error("Couldn't close $self->{_file_path}: $!");
		return undef;
	}
	return 1;
}


sub retrieve {
	my ($self, $sid, $options) = @_;

	$self->File_init($sid, $options);

	# If the session data does not exist, return.
	unless ( -e $self->{_file_path} ) {
		return undef;
	}
	
	unless ( sysopen(FH, $self->{_file_path}, O_RDONLY) ) {
		$self->error("Couldn't open $self->{_file_path}: $!");
		return undef;
	}
	unless (flock(FH, LOCK_SH) ) {
		$self->error("Couldn't lock the file: $!");
		return undef;
	}
	my $data = undef;
	$data .= $_ while <FH>;
	
	close(FH);
	return $self->thaw($data);
}



sub remove {
	my ($self, $sid, $options) = @_;
	
	$self->File_init($sid, $options);
	unless ( unlink ( $self->{_file_path} ) ) {
		$self->error("Couldn't unlink $self->{_file_path}: $!");
		return undef;
	}
	return 1;
}



sub teardown {
	my ($self, $sid, $options) = @_;

	return 1;
}




sub File_init {
	my ($self, $sid, $options) = @_;

	my $file_id = $sid;

	my $dir = $options->[1]->{Directory};
	$self->{_directory} = $dir;
	my $path = File::Spec->catfile($dir, sprintf("$FileName", $file_id));
	$self->{_file_path} = $path;
}


# $self->sweep_objects;
sub sweep_objects
{
	my $self = shift;

	my $dir = $self->{_directory};
	my $filename_pattern = File::Spec->catfile($dir, $FileName);

	$filename_pattern =~ s/\%s/\*/;
	my @files = glob($filename_pattern);
	$filename_pattern =~ s/\*/\(\.\*\)/;
	
	foreach my $file (@files) {
		my ($sid) = ($file =~ /$filename_pattern/);

		# 自分自身はsweepしない
		next if($sid eq $self->id);
		
		my $session = new CGI::Session(
			"driver:FilePP;id:MD5PP;serializer:FreezeThaw", 
			$sid, {Directory=>$dir});

		if($sid ne $session->id) {
		# 期限切れのため、更新されたセッションは、削除
			$session->delete;
		}
	}
}


#=============================================================================
1;	   

