#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

#=============================================================================
# RNA メインオブジェクト
#
# $Id: RNA.pm,v 1.70 2004/01/10 00:42:15 msano Exp $
#=============================================================================
package RNA;

use strict;
use Carp;
use LWP::UserAgent;
# use XML::RSS;
use RNA::Utils qw( :all );
use RNA::RSS;
use RNA::AntennaTree;
use RNA::Hatena;
use RNA::LinkInfo;
use RNA::Plugin;
use RNA::Auth;

use vars qw($VERSION);
my $build_date = join('', ('$Date: 2004/02/08 23:59:00 $' =~ /\d{2}(\d{2})\/(\d{2})\/(\d{2})/));
$VERSION = "1.9.2 (build $build_date)";


sub new {
    my $class = shift;
	my $self = {};
    
    bless $self, $class;
    
    $self->initialize(@_);
    
    return $self;
}

sub initialize {
    my $self = shift;
    my %hash = @_;

    # configuration
    if($hash{Config}){
		require $hash{Config};
		require RNA::Config;
		$self->{Config} = RNA::Config->new;
	}
	else {
		die "Content-type text/plain\n\nConfig File not Found\n";
	}

	$self->{VERSION} = $RNA::VERSION;
	$self->{Config}->{Template}->{Element}->{RNAVersion} = $self->{VERSION};
	
	$self->{Date} = '';

	$self->{MainIndex} = {
		RSS      => {}
	};
	
	$self->{SiteList} = {
		RSS      => {},
		RSSArray => []
	};

	$self->{RecentSiteList} = {
		RSS      => {},
		AntennaTree => {}
	};

	$self->{RecentEntryList} = {
		RSS      => {},
		AntennaTree => {}
	};

	$self->{RecentSiteEntryList} = {
		RSS      => {},
		AntennaTree => {}
	};

	# Trackback ，リンクの情報を格納
	$self->{Cache} = {
		LinkInfo => {} # 	RNA::LinkInfo オブジェクト
	};

	# ログ・オブジェクト
	eval {
		require RNA::Log;

		$self->{Log} = new RNA::Log(
			dir => $self->{Config}->{Log}->{Directory},
			file_size => $self->{Config}->{Log}->{MaxFileSize}, 
			max_files => $self->{Config}->{Log}->{MaxFiles});
	};
#	print STDERR $@ . "\n";

	
	#-------------------------------------------
	# 現在時刻の設定（Dublin Core 形式）
	#-------------------------------------------
	$self->{Date} = &get_current_dc_date;
}


# $html = $self->transfering_page($transfer_uri);
sub transfering_page
{
	my $self = shift;
    my $config = $self->{Config};
	my $transfer_uri = shift;
	require CGI;
	my $header = CGI->new->header(-type=>'text/html', 
		-charset => $config->{Encoding});

	return qq($header
<html>
	<head>
		<title>Refreshing RNA...</title>
		<meta http-equiv="Refresh" content="3;URL=$transfer_uri">
	</head>
	<body bgcolor="#eeeeee" text="#bbbbbb">
		<h2>Refreshing RNA...</h2><br>
		Please wait for 3 second to back, or click [<a href="$transfer_uri">Back</a>].<br>
		It might take a while to check all sites you register.
	</body>
</html>
);
}

#$rna->update;
sub update
{
    my $self = shift;
    my $config = $self->{Config};

	if($config->{Update}->{Background}) {
	# バックグラウンド処理
		my $pid = '';
		$|=1; # 即時出力
		if($pid = fork) {
			# 親プロセス （ブラウザへのレスポンス）
			return 1;
		}
		elsif (defined $pid) {
			# 子プロセス （更新処理）
			close(STDOUT);
			$self->build;
			$self->print;
			exit;
		}
		else {
		# エラー
			&RNA::Utils::die_handler("プロセスを生成できません．\n");
		}	
	}
	else {
	# 通常の処理
		$self->build;
		$self->print;
		return 1;
	}
}

# $rna->run(Mode=>'Update', TransferURI=>$transfer_uri, Response => $redirect);
sub run
{
    my $self = shift;
    my $config = $self->{Config};
	my %hash = @_;

	my $transfer_uri = $hash{TransferURI};
	my $response = $hash{Response};
	my $mode = $hash{Mode};

	if($config->{Update}->{Background}) {
	# バックグラウンド処理
		if($mode eq 'Update') {
			$response = $self->transfering_page($transfer_uri);
		}
	}

	$self->update;

	print $response;
#	close(STDOUT);
}

# $rna->cache(TransferURI=>$transfer_uri, Response => $redirect);
sub cache
{
    my $self = shift;
    my $config = $self->{Config};
	my %hash = @_;

	my $transfer_uri = $hash{TransferURI};
	my $response = $hash{Response};

	if($config->{Update}->{Background}) {
	# バックグラウンド処理
		$response = $self->transfering_page($transfer_uri);

		my $pid = '';
		$|=1; # 即時出力
		if($pid = fork) {
			# 親プロセス （ブラウザへのレスポンス）
			print $response;
			close(STDOUT);
		}
		elsif (defined $pid) {
			# 子プロセス （更新処理）
			close(STDOUT);
			$self->cache_link_info;
			exit;
		}
		else {
		# エラー
			&die_handler("プロセスを生成できません．\n");
		}	
	}
	else {
	# 通常の処理
		$self->cache_link_info;
		print $response;
	}
}

# $self->cache_link_info;
sub cache_link_info
{
	my $self = shift;
	my $config = $self->{Config};

	$self->set_site_list;
	$self->set_rss_array;

	my $rss_array_ref = $self->{SiteList}->{RSSArray};
	my @all_items = ();
	foreach my $r (@{$rss_array_ref}){
		my @items = @{$r->{items}};
		push(@all_items, @items);
	}

	# content:encoded 要素の削除
	foreach my $i (@all_items) {
		delete $i->{content}->{encoded};
	}

	my $current_cache_dir = $config->{Cache}->{CurrentLinkDirectory};
	my $accum_cache_dir = $config->{Cache}->{AccumurativeLinkDirectory};

	# Current Cache
	my $link_info = new RNA::LinkInfo(
		timeout=>$config->{Timeout}, 
		tb_ping_list=>$config->{Cache}->{TrackbackPingURIList});
	$link_info->build(\@all_items);
	$link_info->export_directory($current_cache_dir);

	# Accumurative Cache
	$link_info->reduce;
	$link_info->export_additional($accum_cache_dir);
}

sub build 
{
    my $self = shift;
    my $config = $self->{Config};

	#---------------------------------------------------
	# リストの作成
	#---------------------------------------------------
    # 設定ファイルから，RSS のリストの取り出し
	$self->set_site_list;
	
	$self->set_rss_array;

	#---------------------------------------------------
	# Trackback，リンク情報のインポート
	#---------------------------------------------------
	if($config->{LinkTrace}){
		my $link_info = new RNA::LinkInfo(timeout=>$config->{Timeout}, 
				tb_ping_list=>$config->{Cache}->{TrackbackPingURIList});
		$link_info->import_directory($config->{Cache}->{CurrentLinkDirectory});
		$self->{Cache}->{LinkInfo} = $link_info;
	}

	#---------------------------------------------------
	# Webページの作成
	#---------------------------------------------------

	# index.rdf
	$self->make_index_rss;
	
	# dc:date 順にソートしたサイトのリスト
	$self->make_site_list;

	# 最新記事リストの作成
	$self->make_recent_entry_list;

	# サイトごとの記事リストの作成
	$self->make_recent_site_entry_list;

	#---------------------------------------------------
	# Plugin
	#---------------------------------------------------
	$self->plugin;
}


sub print 
{
	my $self = shift;
	my $config = $self->{Config};


	#----------------------------------------
	# Main Index ページ
	#----------------------------------------
	$self->print_main_index;

	#----------------------------------------
	# RSD
	#----------------------------------------
	$self->print_rsd;

	#----------------------------------------
	# Recent Site リスト
	#----------------------------------------
	{
		my $str = $self->{RecentSiteList}->{RSS}->as_string;
		&print_file($config->{RecentSiteList}->{RSS}, \$str);

		my $antenna = $self->{RecentSiteList}->{AntennaTree};

		my $template = &read_file($config->{Template}->{RecentSiteList});
		my $str_ref = $antenna->apply_template($template, 
			$config->{Template}->{Element});

		&print_file($config->{RecentSiteList}->{HTML}, $str_ref);
	}

	#----------------------------------------
	# Recent Entry リスト
	#----------------------------------------
	{
		my $str = $self->{RecentEntryList}->{RSS}->as_string;
		&print_file($config->{RecentEntryList}->{RSS}, \$str);

		my $antenna = $self->{RecentEntryList}->{AntennaTree};
		my $template = &read_file($config->{Template}->{RecentEntryList});
		my $str_ref = $antenna->apply_template($template, 
			$config->{Template}->{Element});

		&print_file($config->{RecentEntryList}->{HTML}, $str_ref);
	}

	#----------------------------------------
	# Recent Site Entry リスト
	#----------------------------------------
	{
		my $str = $self->{RecentSiteEntryList}->{RSS}->as_string;
		&print_file($config->{RecentSiteEntryList}->{RSS}, \$str);

		my $antenna = $self->{RecentSiteEntryList}->{AntennaTree};
		my $template 
			= &read_file($config->{Template}->{RecentSiteEntryList});
		my $str_ref = $antenna->apply_template($template, 
			$config->{Template}->{Element});

		&print_file($config->{RecentSiteEntryList}->{HTML}, $str_ref);
	}
}


# $self->set_site_list
sub set_site_list
{
    my $self = shift;
    my $config = $self->{Config};
	
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($config->{SiteList}->{File});
	$self->{SiteList}->{RSS} = $list_rss;
}


# $self->set_rss_array;
#
# 最近更新されたページ順に，RSS を配列に入れ，そのリファレンスを返す．
sub set_rss_array
{
	my $self = shift;
	my $list_rss = $self->{SiteList}->{RSS};
	my $config = $self->{Config};
	my $hatena_flag = $config->{HatenaFlag};

	my @rss_array = ();
	my $cached_data    = $self->__cache_sites || {};
	my $new_cache_data = {};
	foreach my $site (@{$list_rss->{items}}){
		my $uri = $site->{link};
		my $subject = $site->{dc}->{subject};

		#------------------------------
		# RSS ドキュメント取得 と Cache の更新
		#------------------------------

		# cache の content が未定義なら、空文字列へのリファレンスを格納
		unless(exists $cached_data->{$uri}->{content}) {
			$cached_data->{$uri}->{content} = \'';
		}

		# 'If-Modified-Since'
		my $if_modified_since = $cached_data->{$uri}->{'Last-Modified'};
		unless(${$cached_data->{$uri}->{content}}) {
			$if_modified_since = '';
		}
#print "$uri\t$if_modified_since\n";
#print ${$cached_data->{$uri}->{content}} . "\n";

		# GET
		my ($str, $headers_ref) 
			= &get_html(uri=>$uri, timeout=>$config->{Timeout}, 
				'If-Modified-Since'=>$if_modified_since);
		# 更新されていない（304 Not modified）場合は、キャッシュのデータを使う
		$str ||= ${$cached_data->{$uri}->{content}} if($cached_data->{$uri});

		# cacheに追加
		$new_cache_data->{$uri} = {content=>\$str, 
			'Last-Modified' => $headers_ref->header('Last-Modified')||$if_modified_since||''};

		# 文字コード変換
		convert_char_code(\$str);
		
		#------------------------------
		# はてなのページを パース
		#------------------------------
		if($subject eq $hatena_flag) {
			my $hatena = new RNA::Hatena(URI => $uri);
			$hatena->parse_html(\$str);
			push (@rss_array, @{$hatena->{RSSArray}});
			next;
		}
		
		# parse
		my $rss = new RNA::RSS;
		if($str =~ /<html.*<\/html>/is && $str !~ /<\!\[CDATA\[.*?\]\]>/s) {
		# HTML
			$rss->parse_html($str);
			$rss->{channel}->{link} ||= $site->{link};
#			$self->log(level=>'debug', message=>"HTML $uri parsed.");
		}
		else {
		# RSS
			$rss->parse($str);
			$rss->set_implicit_element;
#			$self->log(level=>'debug', message=>"RSS $uri parsed.");
		}

		#---------------------------------------------------------
		# 設定ファイルのデータでchannelを上書き
		#---------------------------------------------------------
		$self->_overwrite_channel($rss, $site);

		#---------------------------------------------------------
		# 日付で，<item> をソート
		#---------------------------------------------------------
		my @a = sort 
			{ &comp_dc_date($a->{'dc'}->{'date'}, $b->{'dc'}->{'date'})} 
			@{$rss->{items}};
		$rss->{items} = \@a;

		# HTTP ヘッダ の Last-Modified から日付を取得
		unless ($rss->{channel}->{dc}->{date}) {
			$rss->{channel}->{dc}->{date} = 
				&transform_to_w3cdtf($headers_ref->header('Last-Modified')
						|| $if_modified_since);
		}

		push (@rss_array, $rss);
	}

	# cache したサイトのデータを Store
	$self->__cache_sites($new_cache_data);

	# channel/dc:date の降順にソート
	my @ret = sort
		{&comp_dc_date(
			$a->{channel}->{dc}->{date}, $b->{channel}->{dc}->{date})
		} 
		@rss_array;

	$self->{SiteList}->{RSSArray} = \@ret;
}


# $self->_overwrite_channel($target_rss, $source_channel);
sub _overwrite_channel
{
	my $self = shift;
	my ($trss, $schannel) = @_;
	my $config = $self->{Config};

	# 主な channel データ
	if($config->{SiteList}->{OverwriteChannel}) {
		foreach my $el ('title', 'description'){
			if(!$config->is_empty_content($schannel->{$el})) {
				$trss->{channel}->{$el} = $schannel->{$el};
			}
		}

		# Dublin Core モジュール
		foreach my $el ('creator') {
			if(!$config->is_empty_content($schannel->{dc}->{$el})){
				$trss->{channel}->{dc}->{$el} = $schannel->{dc}->{$el};
			}
		}
	}

	# dc:subject
	if($config->{SiteList}->{OverwriteSubject}) {
		if(!$config->is_empty_content($schannel->{dc}->{subject})){
			$trss->{channel}->{dc}->{subject} = $schannel->{dc}->{subject};
		}
	}

	# ag:source, ag:sourceURL
	if($config->{SiteList}->{OverwriteSource})	{
		foreach my $new_item (@{$trss->{items}}) {
			$new_item->{ag}->{source} = $trss->{channel}->{title};
			$new_item->{ag}->{sourceURL} = $trss->{channel}->{link};
		}
	}
}


# $self->__cache_sites(\%cached_data);
# 	$cached_data{$uri} = {content=>\$str, 'Last-Modified' => $last_modified};
sub __cache_sites
{
	my $self = shift;
	my $hash = shift;
	my $config = $self->{Config};
	my $ext = '.rdf';

	# directory
	my $dir = $config->{Cache}->{SiteDirectory};
	$dir =~ s/\/$//;
	unless(-d $dir){
		return undef;
	}

	my $list_file = $config->{Cache}->{SiteList};

	if($hash) {
	# Store cache of sites
		&clear_directory($dir);

		my $i = 1;
		while(my ($uri, $data) =  each %{$hash}) {
			my $filename = $dir . '/site_' . sprintf("%010d", $i++) . $ext;

			print_file($filename, $data->{content});

			my $last_modified = $data->{'Last-Modified'};
			my $str = "$uri\t$last_modified\t$filename\n";
			print_file($list_file, \$str, '>>');
		}
		return 1;
	}
	else {
	# Read cache list
		my $list_str = eval {read_file($list_file)};

		my $cached_data = {};
		while($list_str =~ s/(.*?)\n//) {
			my ($uri, $last_modified, $filename) = split(/\t/, $1);
			my $str = eval{read_file($filename)};
			$cached_data->{$uri} = {content=>\$str, 
				'Last-Modified'=>$last_modified};
		}

		return $cached_data;
	}
}




# トップページの作成
# $self->print_main_index;
sub print_main_index
{
	my $self = shift;
	my $config = $self->{Config};
	
	my $template = &read_file($config->{Template}->{MainIndex});

	my $encoded_date = $self->{Date};
	my @temp = &RNA::Utils::parse_date_string($encoded_date);
	my $tz_str = $temp[6];

	my %text_hash = %{$config->{Template}->{Element}};
	$text_hash{RNAPageTitle}       = $config->{MainIndex}->{Title};
	$text_hash{RNAPageDescription} = $config->{MainIndex}->{Description};
#	$text_hash{RNAPageDate}        = $self->{Date};

	$text_hash{RNASiteList}        = $config->{SiteList}->{File};
	
	$text_hash{RNARecentSiteRSS}   = $config->{RecentSiteList}->{RSS};
	$text_hash{RNARecentSiteHTML}  = $config->{RecentSiteList}->{HTML};
	$text_hash{RNARecentSiteXML}   = $config->{RecentSiteList}->{XML};

	$text_hash{RNARecentEntryRSS}  = $config->{RecentEntryList}->{RSS};
	$text_hash{RNARecentEntryHTML} = $config->{RecentEntryList}->{HTML};
	$text_hash{RNARecentEntryXML}  = $config->{RecentEntryList}->{XML};

	$text_hash{RNARecentSiteEntryRSS}  = $config->{RecentSiteEntryList}->{RSS};
	$text_hash{RNARecentSiteEntryHTML} = $config->{RecentSiteEntryList}->{HTML};	$text_hash{RNAPageTimezone}    = $tz_str;

	
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}
	# date format
	$template =~ s/(<\$RNAPageDate.*?\$>)/RNA::AntennaTree::apply_template_date($1, $encoded_date)/ges;


	# HTML 出力
	$self->{MainIndex}->{HTML} = $template;
	&print_file($config->{MainIndex}->{File}, \$template);

	# RSS 出力
	my $str = $self->{MainIndex}->{RSS}->as_string;
	&print_file($config->{MainIndex}->{RSS}, \$str);
}

# RSD の作成
# $self->print_rsd;
sub print_rsd
{
	my $self = shift;
	my $config = $self->{Config};

	my $template = q(<?xml version="1.0" ?>
<rsd version="1.0" xmlns="http://archipelago.phrasewise.com/rsd" >
	<service>
		<engineName>RNA <$RNAVersion$></engineName> 
		<engineLink>http://www.semblog.org/</engineLink>
		<homePageLink><$RNABaseURI$></homePageLink>
		<apis>
			<api name="rna" preferred="true" apiLink="<$RNAXMLRPCCGIURI$>" blogID="" />
			<api name="acsh" preferred="true" apiLink="<$RNAFOAFCGIURI$>" blogID="" />
		</apis>
	</service>
</rsd>
);

	my %text_hash = %{$config->{Template}->{Element}};
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	&print_file($config->{RSD}->{File}, \$template);
}


# index.rdf
# $self->make_index_rss;
sub make_index_rss
{
	my $self = shift;
	my $config = $self->{Config};

	#--------------------------------------
	# RSS 作成
	#--------------------------------------
	my $rss = new RNA::RSS;
	&set_rss($rss, 
		$config->{MainIndex}->{Title}, 
		$config->{BaseURI} . $config->{MainIndex}->{File}, 
		$config->{MainIndex}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});

	foreach my $doc (qw(SiteList ClipList)) {
		$rss->add_item(
			title       => $config->{$doc}->{Title},
			link        => $config->{BaseURI} . $config->{$doc}->{RSS},
			description => $config->{$doc}->{Description},
			dc => {
				creator => $config->{Owner},
				subject => $doc
			}
		);
	}

	foreach my $doc (qw(RecentSiteList RecentEntryList RecentSiteEntryList)) {
		$rss->add_item(
			title       => $config->{$doc}->{Title},
			link        => $config->{BaseURI} . $config->{$doc}->{RSS},
			description => $config->{$doc}->{Description},
			dc => {
				date => $self->{Date},
				creator => $config->{Owner},
				subject => $doc
			}
		);
	}

	$self->{MainIndex}->{RSS} = $rss;
}


# dc:date 順にソートしたサイトのリスト
# $self->make_site_list;
sub make_site_list
{
	my $self = shift;
	my $config = $self->{Config};
	my $rss_array_ref = $self->{SiteList}->{RSSArray};

	#--------------------------------------
	# RSS 作成
	#--------------------------------------
	my $rss = new RNA::RSS;
	&set_rss($rss, 
		$config->{RecentSiteList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteList}->{HTML}, 
		$config->{RecentSiteList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});

	foreach my $r (@{$rss_array_ref}){
		$rss->add_item(%{$r->{channel}});
	}

	$self->{RecentSiteList}->{RSS} = $rss;


	#--------------------------------------
	# テンプレートからのHTML作成
	#--------------------------------------
	my $antenna = new RNA::AntennaTree;
	$antenna->initialize($config->{RecentSiteList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteList}->{HTML}, 
		$config->{RecentSiteList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	$antenna->set_from_rss_array($rss_array_ref);

	$self->{RecentSiteList}->{AntennaTree} = $antenna;
}


# 最新記事リストの作成
# $self->make_recent_entry_list;
sub make_recent_entry_list 
{
	my $self = shift;
	my $config = $self->{Config};
	my $rss_array_ref = $self->{SiteList}->{RSSArray};

	my @items = ();
	foreach my $a (@{$rss_array_ref}){
		push (@items, @{$a->{items}} )
	}

	# 日付でソート
	my @a = sort 
		{ &comp_dc_date($a->{'dc'}->{'date'}, $b->{'dc'}->{'date'})} 
		@{items};
	splice @a, $config->{RecentEntryList}->{Number};
	my $items_ref = \@a;

	#---------------------------------------------------
	# RSS の作成
	#---------------------------------------------------
	my $rss = new RNA::RSS;
	&set_rss($rss, 
		$config->{RecentEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentEntryList}->{HTML}, 
		$config->{RecentEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	foreach my $item (@{$items_ref}){
		$rss->add_item(%{$item});
	}

	$self->{RecentEntryList}->{RSS} = $rss;

	#--------------------------------------
	# テンプレートからのHTML作成
	#--------------------------------------
	my $antenna = new RNA::AntennaTree;
	$antenna->initialize($config->{RecentEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentEntryList}->{HTML}, 
		$config->{RecentEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	$antenna->set_from_items_array($items_ref);

#print STDERR (keys %{$self->{Cache}->{LinkInfo}->link_info()})[0] . "\n";
	# Trackback，リンク情報
	if($config->{LinkTrace}){
		$antenna->add_link_info($self->{Cache}->{LinkInfo}->link_info);
	}

	$self->{RecentEntryList}->{AntennaTree} = $antenna;
}


# サイトごとの記事リストの作成
# $self->make_recent_site_entry_list;
sub make_recent_site_entry_list
{
	my $self = shift;
	my $config = $self->{Config};
	my $rss_array_ref = $self->{SiteList}->{RSSArray};

	#--------------------------------------
	# HTML作成
	#--------------------------------------
	my $antenna = new RNA::AntennaTree;
	$antenna->initialize($config->{RecentSiteEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteEntryList}->{HTML},
		$config->{RecentSiteEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	$antenna->set_from_rss_array($rss_array_ref);

	foreach my $site (@{$antenna->{items}}){
		my $configured_num = $config->{RecentSiteEntryList}->{NumberPerSite};
		my $num_per_site = 
			$configured_num < scalar(@{$site->{items}}) ? 
				$configured_num : scalar(@{$site->{items}});
		splice @{$site->{items}}, $num_per_site;
	}

	# Trackback，リンク情報
	if($config->{LinkTrace}){
		$antenna->add_link_info($self->{Cache}->{LinkInfo}->link_info);
	}

	$self->{RecentSiteEntryList}->{AntennaTree} = $antenna;

	#---------------------------------------
	# RSS 
	#---------------------------------------
	my $rss = new RNA::RSS;
	&set_rss($rss, $config->{RecentSiteEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteEntryList}->{HTML}, 
		$config->{RecentSiteEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});

	my @all_items = ();
	foreach my $r (@{$rss_array_ref}){
		my $configured_num = $config->{RecentSiteEntryList}->{NumberPerSite};
		my $num_per_site = 
			$configured_num < scalar(@{$r->{items}}) ? 
			$configured_num : scalar(@{$r->{items}});
		my @contents = @{$r->{items}};
		splice @contents, $num_per_site;
		push(@all_items, @contents);
	}

	$rss->{items} = \@all_items;

	$self->{RecentSiteEntryList}->{RSS} = $rss;
}


# Plugin
# $self->plugin;
sub plugin
{
	my $self = shift;

	my $config = $self->{Config};
	my $plugin_root = $config->{Plugin}->{RootDirectory};

	return 0 if(!$plugin_root);

	my $log_name = 'plugin';
	# ログのエントリ
	$self->add_log(name=>$log_name);

	my $ret_dir = eval {require Cwd; Cwd::cwd();} || '../..';
	foreach my $dir (glob("$plugin_root/*")){
		$dir =~ s/\/$//;

		# main script file
		my @path = split(/\//, $dir);
		my $main_script = (pop @path) .  '.pl';
		
		RNA::Plugin::initialize(
			Directory=>$dir, RNA=>$self, RNADirectory=>$ret_dir);

		# change directory to each plugin directory
		chdir($dir);

		my $err = RNA::Plugin::execute($main_script);

		# change directory to RNA root
		chdir($ret_dir);

		# Log
		$self->log(name=>$log_name, message=>($err ? $err : "Plugin '$main_script' is Done."));
	}

	return 1;
}


# 認証
#
# $bool = $rna->authenticate(cgi=>$cgi);
#	$cgi : CGI オブジェクト
sub authenticate
{
	my $self = shift;
	my $config = $self->{Config};

	my %hash = @_;
	my $cgi = $hash{cgi} || eval {require CGI; CGI->new};

	my $passwd_file = $config->{Authentication}->{PasswordFile};
	my $auth = new RNA::Auth(file=>$passwd_file);
	return $auth->check_cookie($cgi) || $auth->check_param($cgi);
}


# Javascript 出力
#
# $self->js_output('template_uri'=>'http://hogehoge/template1, 
#		target=>'RecentEntryList', encoding=>'UTF-8');
#
#	target: 'RecentEntryList', 'RecentSiteList', 
#			'RecentSiteEntryList', 'ClipList'
#
sub js_output
{
	my $self = shift;
	my $config = $self->{Config};

	my %arg = @_;
	my $target = $arg{target}||'RecentEntryList';
	my $template_uri = $arg{'template_uri'}
		||$config->{BaseURI} . $config->{Template}->{JavascriptDefault};
	my $encoding = $arg{encoding}||$config->{Encoding};

	my ($template, $headers) 
		= &get_html(uri=>$template_uri, timeout=>$config->{Timeout});

	my $rss = new RNA::RSS;
	$rss->parsefile($config->{$target}->{RSS});
	my $str_ref = RNA::AntennaTree::apply_template_to_rss(
						$rss, $template, $config->{Template}->{Element});

	# 文字コード変換
	&convert_char_code($str_ref, $encoding);

	# 出力
	require CGI;
	print CGI->new->header(
		-type=>'application/x-javascript', -charset=>$encoding);
	$$str_ref =~ s/'/\\'/g;
	$$str_ref =~ s/\r\n|\n//gs;
	my @out = split(/ript>/, $$str_ref);
	my $ret = '';
	foreach my $i (0 .. $#out) {
		my $str = $out[$i];
		if($i > 0) {
			$str = 'pt>' . $str;
		}
		if($i < $#out) {
			$str .= 'ri';
		}
		$ret .= "document.write('$str')\;\n";
	}

	return $ret;
}


# $self->log(level=>'info', message=>"abcdefg" );
# $self->log(name=>'hogehoge', message=>"abcdefg" );
sub log
{
	my $self = shift;
	my %arg = @_;

	if($arg{name}) {
		$arg{level} = 'emerg';
		eval{ $self->{Log}->log_to(%arg); };
	}
	elsif($arg{level}) {
		eval{ $self->{Log}->log(%arg); };
	}
	else {
		croak 'Invalid log argument';
	}
}

# $self->add_log(name=>'name');
sub add_log 
{
	my $self = shift;
	my %arg = @_;

	eval {
		$self->{Log}->add(
			name=>$arg{name}, min_level=>'emerg', max_level=>'emerg');
	};
}

#==============================================================================
1;
__END__

