package RNA::Auth;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# Authentication Module
# 　
#=============================================================================

use strict;
use Carp;
use RNA::Auth::File;
use RNA::Utils;
use vars qw($VERSION);

$VERSION = '0.1';

my $default_encoding = 'UTF-8';
my $username_label = 'username';
my $passwd_label = 'passwd';
my $default_template = q(<html>
	<head>
		<title>RNA login</title>
		<meta http-equiv="Content-Type" content="text/html; charset=)
		. $default_encoding
		. q(" />
		<meta http-equiv="Content-Script-Type" content="text/javascript" />
		<script>
		<!--
		function set_cookie(<$RNAUserNameLabel$>, <$RNAPasswdLabel$>) {
			document.cookie
				 = "<$RNAUserNameLabel$>=" + escape(<$RNAUserNameLabel$>) + "; ";
			//	+ "path=/home/username/public_html/rna/; ";
			document.cookie
				 = "<$RNAPasswdLabel$>=" + escape(<$RNAPasswdLabel$>) + "; ";
		}
		// -->
		</script>
	</head>

	<body bgcolor="#eeeeee" text="#bbbbbb">
		<h2>RNA Login</h2><br>

		<form method="post" action="<$RNATransferURI$>" onSubmit="set_cookie(this.<$RNAUserNameLabel$>.value, this.<$RNAPasswdLabel$>.value); return true;">
		<table>
		<tr><td><$RNAUserNameLabel$> :</td><td><input type="text" name="<$RNAUserNameLabel$>"></td></tr>
		<tr><td><$RNAPasswdLabel$> :</td><td><input type="password" name="<$RNAPasswdLabel$>"></td></tr>
		<tr><td><input type="submit" value="Login"></td></tr>
		</form>
	</body>
</html>
);

# $auth = new RNA::Auth(file=>'.rna-passwd');
sub new {
	my $class = shift;
	my $self = {};

	bless $self, $class;
    
	$self->init(@_);
	
	return $self;
}


sub init {
    my $self = shift;
    my %hash = @_;

	$hash{file} || croak "AuthFile is required.";
	$self->{_driver} = new RNA::Auth::File(file=>$hash{file});
}


# $bool = $self->check($username, $passwd);
sub check
{
	my $self = shift;
	return $self->{_driver}->check(@_);
}

# $bool = $self->add_user($username, $passwd);
sub add_user
{
	my $self = shift;
	return $self->{_driver}->add_user(@_);
}

# $bool = $self->update_user($username, $passwd);
sub update_user
{
	my $self = shift;
	return $self->{_driver}->update_user(@_);
}

# $bool = $self->delete_user($username);
sub delete_user
{
	my $self = shift;
	return $self->{_driver}->delete_user(@_);
}


# $bool = $self->check_cookie($cgi);
sub check_cookie
{
	my $self = shift;
	my $cgi = shift;

	my $username = $cgi->cookie($username_label);
	my $passwd = $cgi->cookie($passwd_label);

	return $self->check($username, $passwd);
}


# $bool = $self->check_cookie($cgi);
sub check_param
{
	my $self = shift;
	my $cgi = shift;

	my $username = $cgi->param($username_label);
	my $passwd = $cgi->param($passwd_label);

	return $self->check($username, $passwd);
}


# $label = &RNA::Auth::username_label;
sub username_label
{
	return $username_label;
}

# $label = &RNA::Auth::passwd_label;
sub passwd_label
{
	return $passwd_label;
}



# $str = $self->login_page(transfer=>$uri, template=>$template, encoding=>'UTF-8');
sub login_page
{
	my $self = shift;
	my %hash = @_;

	my $template = $hash{template} || $default_template;
	my $transfer_uri = $hash{transfer};
	my $encoding = $hash{encoding} || $default_encoding;

	my %text_hash = (
		RNAUserNameLabel=>$username_label, RNAPasswdLabel=>$passwd_label, 
		RNATransferURI=>RNA::Utils::html_encode_minimal($transfer_uri), 
		RNAEncoding=>$encoding);

	foreach my $t (keys %text_hash){
		$template =~ s/<\$$t\$>/$text_hash{$t}/ges;
	}

	require CGI;
	my $header = CGI->new->header(-type=>'text/html', -charset => $encoding);

	return $header . $template;
}



#=============================================================================
1;
__END__

