package RNA::Log;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# 　Log 出力用モジュール
#=============================================================================

use strict;
use Carp;
use Log::Dispatch;               # require 5.005
use Log::Dispatch::FileRotate;   # require 5.005

use vars qw($VERSION);

$VERSION = '0.1';


# new RNA::Log(dir=>'cache/log', file_size=>1024*1024, max_files => 6);
sub new {
	my $class = shift;
	my $self = {};

	bless $self, $class;
    
	$self->init(@_);

	return $self;
}


sub init {
    my $self = shift;
    my %hash = @_;

	# Directory
	$hash{dir} =~ s/\/$//;
	$self->{dir} = $hash{dir}||'.';

	$self->{file_size} = $hash{file_size};  # [Bytes]
	$self->{max_files} = $hash{max_files};
	$self->{dispatcher} = Log::Dispatch->new;

	$self->clear_items;

	# デフォルトのログ
	$self->add(name => 'debug', max_level => 'debug');
	$self->add(name => 'info', min_level => 'info', max_level=>'notice');
	$self->add(name => 'error', min_level => 'warning');

#print STDERR %{$self->{dispatcher}->output('error')}; 
}

sub clear_items {
	my $self = shift;

	foreach my $item (keys %{$self->{items}}) {
		$self->{dispatcher}->remove($item->{name});
	}

	$self->{items} = {};
}

# $self->add(name => 'main', min_level => 'error', max_level=>'emerg');
#
# log level:
# debug info notice warning error(err) critical(crit) alert emergency(emerg)
sub add
{
	my $self = shift;
	my %arg  = @_;
	my $name = $arg{name};
	my $min_level = $arg{min_level}||'debug';
	my $max_level = $arg{max_level}||'emerg';

	return undef unless($name);
	
	$self->{items}->{$name} 
		= {
			name      => $name,
			min_level => $min_level,
			max_level => $max_level,
			filename  => $self->{dir} . '/' . $name . '.log',
			mode      => 'append' ,
			size      => $self->{file_size},
			max       => $self->{max_files},
#			DEBUG => 1,
		};

	$self->{dispatcher}->add(
		Log::Dispatch::FileRotate->new(%{$self->{items}->{$name}})
	);
}


# $self->log(level=>'info', message=>"abcdefg" );
sub log
{
	my $self = shift;
	my %arg  = @_;

	$arg{message} = $self->make_message($arg{message});

	$self->{dispatcher}->log(%arg);
}


# $self->log_to(name=>'name', message=>"abcdefg" );
sub log_to
{
	my $self = shift;
	my %arg  = @_;

	$arg{message} = $self->make_message($arg{message});

	$self->{dispatcher}->log_to(%arg);
}


# $msg = $self->make_message($str);
sub make_message
{
	my $self = shift;
	my $str  = shift;

	chomp($str); chomp($str);

	my $msg = '[' . $self->timestamp . '] ' ;
	if($ENV{'REMOTE_HOST'}||$ENV{'REMOTE_ADDR'}) {
		$msg .= '[client ' . ($ENV{'REMOTE_HOST'}||$ENV{'REMOTE_ADDR'}) . '] ';
	}
	$msg .= "$str\n";

	return $msg;
}


# $self->timestamp;
sub timestamp
{
	my $self = shift;

	return localtime();
}

#==============================================================================
1;
__END__


#!/usr/bin/perl
#=============================================================================
#----------------------------------------------------------------------------
# デバッガ
#----------------------------------------------------------------------------

use strict;

use lib qw(lib extlib);
use RNA::Log;

my $log = new RNA::Log(dir=>'cache/log', file_size=>40, max_files => 6);

$log->log( level => 'info', message => "abcdefg" );
$log->log( level => 'error', message => "zzzxxx" );


