package RNA::Plugin;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
#
# Plugin
#=============================================================================

use strict;
use RNA::AntennaTree;
use RNA::Utils qw( :all );

use vars qw($VERSION);
$VERSION = '0.1';

my $RNA = {};
my $Directory = '';
my $RNADirectory = '';

# RNA::Plugin::initialize(
#	RNA=>$rna, Directory=>$plugin_dir, RNADirectory=>$rna_dir);
sub initialize {
    my %hash = @_;

	$RNA = $hash{RNA};
	$Directory = $hash{Directory};
	$Directory =~ s/\/$//;

	$RNADirectory = $hash{RNADirectory};
	$RNADirectory =~ s/\/$//;
}

# $error_msg = RNA::Plugin::execute($file);
sub execute
{
	my $file = shift;
	my $result = eval {require $file;};
#print STDERR $@ . "\n";
#print STDERR Cwd::cwd() . "\n";
#print STDERR $file . "\n";

	return $@;
}

# RNA::Plugin::get_rna_uri
sub get_rna_uri  {return $RNA->{Config}->{BaseURI};}

# RNA::Plugin::get_rna_dir
sub get_rna_dir  {return $RNADirectory;}

# RNA::Plugin::get_rna_view_dirname
sub get_rna_view_dirname  {return $RNA->{Config}->{ViewDirectory};}

# RNA::Plugin::get_my_dir
#
#	relative path of current directory
sub get_my_dir  {return $Directory;}

# RNA::Plugin::get_my_dir_uri
sub get_my_dir_uri {return $RNA->{Config}->{BaseURI} . $Directory . '/';}

# RNA::Plugin::get_owner_name
sub get_owner_name {return $RNA->{Config}->{Owner};}

# RNA::Plugin::add_template_element(VarName => 'Value');
sub add_template_element
{
	my %hash = @_;
	
	my $template_hash_ref = $RNA->{Config}->{Template}->{Element};

	my $ret = 1;
	while(my ($k, $v) = each %hash) {
#print STDERR "$k -> $v\n";

		if(exists $template_hash_ref->{$k}) {
			$ret = 0;
		}
		else{
			$template_hash_ref->{$k} = $v;
		}
	}

	return $ret;
}

# RNA::Plugin::get_recent_site_rss;
sub get_recent_site_rss {return $RNA->{RecentSiteList}->{RSS};}
sub get_recent_entry_rss {return $RNA->{RecentEntryList}->{RSS};}
sub get_recent_site_entry_rss {return $RNA->{RecentSiteEntryList}->{RSS};}

# template
# $str = RNA::Plugin::apply_template_to_rss($rss, $template_file)
sub apply_template_to_rss
{
	my ($rss, $template_file) = @_;
	my $config = $RNA->{Config};
	
	my $template = read_file($template_file);
	my $str_ref = RNA::AntennaTree::apply_template_to_rss($rss, 
		$template, $config->{Template}->{Element});

	return $$str_ref;
}

# template
# $str = RNA::Plugin::decode_template($template_file)
sub decode_template
{
	my ($template_file) = @_;
	my $config = $RNA->{Config};
	
	my $template = read_file($template_file);
	my %text_hash = %{$config->{Template}->{Element}};

	# preprocess
	foreach my $t (keys %text_hash){
		if($t =~ /^__RNA/){
			$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
		}
	}
	# apply
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	return $template;
}



#=============================================================================
1;
__END__

