#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# 　Session管理モジュール
#=============================================================================
package RNA::Session;

use strict;
use Carp;

use CGI::Session qw/-ip-match/;
use CGI::Session::ID::MD5PP;

use vars qw($VERSION);
$VERSION = '0.1';


# CGI::Session クラスの変数
$CGI::Session::NAME = 'RNASESSIONID';

my $DRIVER = 'FilePP';
my $ID_GENERATOR = 'MD5PP';
my $SERIALIZER = 'FreezeThaw';

# CGI::Session の method の読み込み
foreach my $m (qw(param id close delete expire)) {
	eval "sub $m \{my \$s=shift\; return \$s->\{_session\}->$m\(\@_\)\;}";
}

# $session = new RNA::Session(cgi=>$cgi, dir=>$dir);
# $session = new RNA::Session(id=>$sess_id, dir=>$dir);
sub new 
{
	my $class = shift;
	my $self = {};
	my %hash  = @_;

	bless $self, $class;

	$self->init(%hash);

	return $self;
}


sub init {
	my $self = shift;
	my %hash = @_;

	my $dir = $hash{dir} || &default_dir 
		|| croak "Writable directory is required.";
	my $cgi_or_id = $hash{cgi} || $hash{id} || undef;
	
	$self->{_session} = 
		new CGI::Session(
			"driver:$DRIVER;id:$ID_GENERATOR;serializer:$SERIALIZER", 
			$cgi_or_id, {Directory=>$dir});

	# 期限切れセッションのsweep
#	$self->sweep_objects;
}

# $self->sweep_objects
sub sweep_objects
{
	my $self = shift;

	$self->{_session}->sweep_objects;
}

# $bool = RNA::Session::default_dir;
sub default_dir
{
	return 
		&check_dir('/tmp')     ? '/tmp'     : 
		&check_dir($ENV{TMP})  ? $ENV{TMP}  :
		&check_dir($ENV{TEMP}) ? $ENV{TEMP} : 
		&check_dir('./tmp')    ? './tmp'    : 
		&check_dir('.')        ? '.'        : 
		undef;
}

# $bool = RNA::Session::check_dir($dir);
#	true : can use the directory for session object 
sub check_dir
{
	my $dir = shift;
	$dir =~ s/\/$//;

	my $ret = undef;
	my $tmpfile = "$dir/" . CGI::Session::ID::MD5PP::generate_id() . '.tmp';
	if(open(FILE, ">$tmpfile")) {
		$ret = 1;
	}
	else {
		$ret = undef;
	}
	close(FILE);
	unlink($tmpfile);

	return $ret;
}


#==============================================================================
1;

