#=============================================================================
# Copyright (C) 2003,2004           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

#=============================================================================
# ユーティリティ
#=============================================================================
package RNA::Utils;
use strict;

use LWP::UserAgent;
use CGI::Carp;
use HTTP::Date qw(time2str str2time time2iso time2isoz);
use Time::Zone;

use Exporter;
@RNA::Utils::ISA = qw( Exporter );
use vars qw( @EXPORT_OK @EXPORT %EXPORT_TAGS $TIME_ZONE $ENCODE_MODULE);
# @EXPORT = qw();
my @simbol_list = 
	qw( get_current_dc_date transform_to_dc_date transform_to_w3cdtf
		calc_day_of_week dc_date 
		dcdate_to_rfc822 parse_date_string dc2ymd 
		diff_dc_date comp_dc_date format_date
		set_rss print_file read_file 
		get_html html_encode html_encode2 html_encode_minimal html_decode 
		die_handler url_encode url_decode
		clear_directory convert_char_code);
@EXPORT_OK = @simbol_list;
%EXPORT_TAGS = (all => [@simbol_list]);

# time zone
BEGIN {
	my $offset = &tz_offset;
	my $sign = $offset >= 0 ? '+' : '-';
	my $tz = $sign . sprintf("%02d:00", abs($offset) / 60 / 60);
	$TIME_ZONE = $tz;
}

# charset 
BEGIN {
	eval {require Encode;};
	if($@) {
	# use Jcode.pm
		require Jcode;
		$ENCODE_MODULE = 0;
	}
	else{
	#use Encode.pm
		import Encode qw(from_to);
		require Encode::Guess;
		require Encode::Alias;
		$ENCODE_MODULE = 1;
	}
}


#-----------------------------------------------------------------------------
# 
#-----------------------------------------------------------------------------


# &set_rss($rss, $title, $link, $description, $date, $creator, $language);
sub set_rss
{
	my ($rss, $title, $link, $description, $date, $creator, $language) = @_;

	$rss->channel(
		title 		=> $title,
		link		=> $link,
		description	=> $description,
		dc => {
			date       => $date,
			creator    => $creator,
			language   => $language
		},
	);
	
	$rss->add_module(prefix=>'cc', uri=>"http://web.resource.org/cc/");
}



# ($html, $headers) = &get_html(uri=>$uri, timeout=>$sec, 
#		'If-Modified-Since'=>$date);
#
# $headers : HTTP::Headers
sub get_html
{
	my %arg = @_;
	my $uri = $arg{uri};
	my $timeout = $arg{timeout};
	my $if_modified_since = $arg{'If-Modified-Since'};

	my $req = HTTP::Request->new(GET => $uri);
#	$req->header('Accept' => 'text/html');

	# 'If-Modified-Since' を指定
	if($if_modified_since) {
		$req->header('If-Modified-Since' => $if_modified_since);
	}
	
	# authorization
#	$req->authorization_basic('user', 'passwd');

	# send request 
	my $ua = LWP::UserAgent->new;
	$ua->agent("RNA/" . $ua->agent);
	if($timeout) { # timeout
		$ua->timeout($timeout);
	}
	$ua->parse_head(0); # HTML::HeadParser を必要としないように  
	my $res = $ua->request($req);

	# http header
	my $headers = $res->headers();

	# check
	if ($res->is_success) {
#print $res->content;
		return ($res->content, $headers);
	}
	else {
#		print "Error: " . $res->status_line . "\n";
		return ('', $headers);
	}
}


# clear_directory($dir)
sub clear_directory
{
	my $dir = shift;
	$dir =~ s/\/$//;
	$dir =~ s/\\$//;

	unlink(glob($dir . '/*'));
}

#-----------------------------------------------------------------------------
# 日付の計算
#-----------------------------------------------------------------------------

# W3CDTF
# $dc_date = get_current_dc_date;
sub get_current_dc_date
{
#	my ($s,$m,$h,$md,$M,$y,$wd,$yd,$isdst) = localtime(time);
#	my $month = ($M + 1);
#	$y += 1900;
#	return &dc_date($y,$month,$md,$h,$m,$s, $TIME_ZONE);

	my $iso_date = &time2iso(time);
	$iso_date =~ s/ /T/;
	return $iso_date . $TIME_ZONE;
}

# Auto detect of date format and transformation to W3CDTF format
#
# $w3cdtf_date = &transform_to_w3cdtf($unknown_date_format)
sub transform_to_w3cdtf
{
	my $str = shift;
	my $time = str2time($str);
	return undef unless $time;

	# ローカルの TIME ZONE で返す
	my $iso = HTTP::Date::time2iso($time);
	$iso =~ s/\s+/T/;
	return $iso . $TIME_ZONE;

#	my ($year, $month, $day, $hour, $min, $sec, $tz) 
#		= HTTP::Date::parse_date($str);
#	if($tz =~ /([-+])(\d+):(\d+)/) {
#		# no
#	}
#	else {
#		my $offset = &tz_offset($tz);
#		$tz = sprintf("%02d:00", $offset / 60 / 60);
#		$tz = '+' . $tz if ($offset >= 0);
#	}
#
#	return &dc_date($year, $month, $day, $hour, $min, $sec, $tz);
}


# Auto detect of date format and transformation to Dublin Core format
# (※ Old routine.)
#
# $dc_date = &transform_to_dc_date($unknown_date_format)
sub transform_to_dc_date
{
	my $date = shift;

	return transform_to_w3cdtf($date);

#	my $dc_date = '';

#	if ($date =~ /(\w+?,\s*)?\d+?\s+?\w+?\s+?\d+?\s+?\d+?:\d+?(:\d+?)?\s*?/) {
#	# RFC822     Sun, 29 Sep 2002 19:59:01 GMT
#		$dc_date = &rfc822date_to_dc($date);
#	}
#	else{
#	# Dublin Core format
#		$dc_date = $date;
#	}
#
#	return $dc_date;
}

# ($wday) = calc_day_of_week($y, $m, $d);
#
# ツェラーの公式
#
# $wday ---  0:日曜日, 1:月曜日, 2:火曜日,3:水曜日, 4:木曜日, 5:金曜日,6=土曜日
#
sub calc_day_of_week
{
	my ($y, $m, $d) = @_;
	my $wday = 0;

	if ($m == 1 || $m == 2){ 
		$y--; 
		$m += 12;
	}
	
	$wday = ($y + int($y/4) - int($y/100) + int($y/400)
				+ int(2.6*$m+1.6) + $d) % 7;

	return $wday;
}


# 2003-05-10T22:57:49 
# $str = &dc_date($year, $month, $day, $hour, $minute, $second, $zone)
sub dc_date
{
	my ($year, $month, $day, $hour, $minute, $second, $zone) = @_;
	return sprintf("%4d-%02d-%02dT%02d:%02d:%02d%s",
		$year,$month,$day,$hour,$minute,$second, $zone);
}

# ※ 削除予定 ※
#
# $dc_date = &rfc822date_to_dc($rfc822date);
#
# http://asg.web.cmu.edu/rfc/rfc822.html#sec-5
#    e.g.)	Sun, 29 Sep 2002 19:59:01 GMT
#
#sub rfc822date_to_dc
#{
#	my $date = shift;
#	chomp($date);
#	
#	my %month_hash = (
#		Jan => '01', Feb=>'02', Mar=>'03', Apr=>'04', May=>'05', Jun=>'06', 
#		Jul=>'07', Aug=> '08', Sep=>'09', Oct=>'10', Nov=>'11', Dec=>'12');
#
#	my ($wd_str, $ymdhmsz) = split(/\s*,\s*/, $date);
#	unless($ymdhmsz) {
#		$ymdhmsz = $wd_str;
#	}
#	my ($day, $m_str, $year, $time, $zone) = split(/\s+/, $ymdhmsz);
#	my $month = $month_hash{$m_str};
#
#	# 00:00:00
#	my ($h, $m, $s) = split(/:/, $time);
#
#	# zone
#	my %zone_hash = (UT=>'', GMT=>'', EST=>'-05:00', EDT=>'-04:00', 
#		CST=>'-06:00', CDT=>'-05:00', MST=>'-07:00', MDT=>'-06:00', 
#		PST=>'-08:00', PDT=>'-07:00', 
#		Z=>'', A=>'-01:00', , B=>'-02:00', C=>'-03:00', D=>'-04:00', 
#		E=>'-05:00', F=>'-06:00', G=>'-07:00', H=>'-08:00', I=>'-09:00', 
#		K=>'-10:00', L=>'-11:00', M=>'-12:00', N=>'+01:00', O=>'+02:00', 
#		P=>'+03:00', Q=>'+04:00', R=>'+05:00', S=>'+06:00', T=>'+07:00', 
#		U=>'+08:00', V=>'+09:00', W=>'+10:00', X=>'+11:00', Y=>'+12:00');
#
#	my $zones = join('|', keys %zone_hash);
#	my $z = '';
#	if($zone =~ /^($zones)$/) {
#		$z = $zone_hash{$1};
#	}
#	elsif($zone =~ /(\-|\+)(\d{2})(\d{2})/) {
#		$z = $1 . $2 . ':' . $3;
#	}
#
#	return sprintf("%4d-%02d-%02dT%02d:%02d:%02d%s",
#		$year,$month,$day,$h,$m,$s, $z);
#}

# $rfc822date = &dcdate_to_rfc822($dc_date);
#
# http://asg.web.cmu.edu/rfc/rfc822.html#sec-5
#    e.g.)	Sun, 29 Sep 2002 19:59:01 GMT
#
sub dcdate_to_rfc822
{
	my $dc_date = shift;
	my ($y, $M, $d, $h, $m, $s, $z) = &parse_date_string($dc_date);

	my %month_hash = (
		'01' => 'Jan', '02'=>'Feb', '03'=>'Mar', '04'=>'Apr', 
		'05'=>'May', '06'=>'Jun', '07'=>'Jul', '08'=> 'Aug', 
		'09'=>'Sep', '10'=>'Oct', '11'=>'Nov', '12'=>'Dec');

	# month
	my $month = $month_hash{sprintf("%02d", $M)};

	# calc  day-of-week (Zeller)
	my @wday_ar = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
	my $wday = &calc_day_of_week($y, $M, $d);
	my $wday_str = $wday_ar[$wday];

	# time zone   +0900
	$z =~ s/://;
	
	return sprintf("%s, %02d %s %s %02d:%02d:%02d %s", 
		$wday_str, $d, $month, $y, $h, $m, $s, $z);
}


# ($y, $M, $d, $h, $m, $s, $z) = &parse_date_string($dc_date);
#
# $z : +09:00   -04:00 など
sub parse_date_string
{
	my ($year, $month, $day, $hour, $min, $sec, $tz) 
		= HTTP::Date::parse_date(shift);

	return ($year, $month, $day, $hour, $min, $sec, $tz);
}


sub dc2ymd
{
	my $dc = shift;

	unless($dc) {
		return '0';
	}

	my ($ymd, $time) = split(/T/, $dc);
	my ($y, $M, $d) = split(/-/, $ymd);
	$time =~ /(\d+):(\d+):(\d+)(.*)/i;
	my ($h, $m, $s, $z) = ($1, $2, $3, $4);

	my $ret_value = sprintf("%4d%02d%02d%02d%02d%02d", $y, $M, $d, $h, $m, $s);
	

	# タイムゾーンに関して
	if($z =~ /([-+])(\d+):(\d+)/i){
		my ($zf, $zh, $zm) = ($1, $2, $3);

		my $diff = sprintf("%02d%02d00", $zh, $zm);
		if($zf eq '-') {
			$ret_value += $diff;
		}
		else {
			$ret_value -= $diff;
		}
	}

	return $ret_value;
#	return ($y, $M, $d, $h, $m, $s);
}

# 2003-05-10T22:57:49+09:00
# $nday = &diff_dc_date($dc_date1, $dc_date2);
# 	$dc_date1 < $dc_date2  :  -
# 	$dc_date1 == $dc_date2 :  0 
# 	$dc_date1 > $dc_date2  :  +, 
sub diff_dc_date
{
	my($dc_date1, $dc_date2) = @_;

#	($y1, $M1, $d1, $h1, $m1, $s1) = &dc2ymd($dc_date1);
#	($y2, $M2, $d2, $h2, $m2, $s2) = &dc2ymd($dc_date2);
#	my $d1 = sprintf("%4d%02d%02d%02d%02d%02d", $y1, $M1, $d1, $h1, $m1, $s1);
#	my $d2 = sprintf("%4d%02d%02d%02d%02d%02d", $y2, $M2, $d2, $h2, $m2, $s2);

	my $d1 = &dc2ymd($dc_date1);
	my $d2 = &dc2ymd($dc_date2);

	return $d1 - $d2;
}

# 2003-05-10T22:57:49+09:00
# $result = &comp_dc_date($dc_date1, $dc_date2);
# 	$dc_date1 < $dc_date2  :  -
# 	$dc_date1 == $dc_date2 :  0 
# 	$dc_date1 > $dc_date2  :  +, 
sub comp_dc_date
{
	my($dc_date1, $dc_date2) = @_;
	return &dc2ymd($dc_date2) <=> &dc2ymd($dc_date1);
}


# $formated_str = &format_date($format_str, $date_str);
#    %a : abbreviated weekday    (e.g. Mon, Thu)
#    %A : weekday                (e.g.Monday, Thursday)
#    %b : abbreviated month name (e.g.Jan, Sep)
#    %B : month name.            (e.g. September)
#    %d : day of the month, two digits, (e.g. 09, 01)
#    %e : day of the month,      (e.g. ' 9') 
#    %H : 24-hour clock.         (e.g. 16, 03) 
#    %I : 12-hour clock,         (e.g. 04, 11)
#    %j : day of the year.       (e.g. 056)       * not implimented 
#    %k : 24-hour clock.         (e.g. ' 9')
#    %l : 12-hour clock          (e.g. ' 4'). 
#    %m : month, two digits,     (e.g. 09) 
#    %M : minute, two digits.    (e.g. 02)
#    %p : Either AM or PM        (e.g. AM)
#    %S : second, two digits,    ( e.g. 04)
#    %x : language-aware date representation.    * not implemented
#    %X : language-aware time representation.    * not implemented
#    %y : year, two digits,      (e.g. 01) 
#    %Y : year, four digits.     (e.g. 2001)
sub format_date
{
	my ($form_str, $date_str) = @_;
	my @wdays 
		= qw( Sunday Monday Tuesday Wednesday Thursday Friday Saturday );
	my @months = qw( January February March April May June
				July August September October November December );
	my @ampm = qw(AM PM);

	# parse date
	my ($year, $month, $day, $hour, $min, $sec, $tz) 
		= HTTP::Date::parse_date($date_str);

	my $wday = &calc_day_of_week($year, $month, $day);

	# hash に割り当て
	my %conv = (
		a => substr($wdays[$wday], 0, 3), 
		A => $wdays[$wday],
		b => substr($months[$month - 1], 0, 3),
		B => $months[$month - 1],
		d => sprintf("%02d", $day),
		e => sprintf("%2d", $day),
		H => sprintf("%02d", $hour),
		I => sprintf("%02d", $hour % 12),
		#   %j : day of the year.       (e.g. 056)       * not implimented 
		k => sprintf("%2d", $hour),
		l => sprintf("%2d", $hour % 12), 
		m => sprintf("%02d", $month), 
		M => sprintf("%02d", $min),
		p => $ampm[int($hour / 12)],
		S => sprintf("%02d", $sec), 
		#    %x : language-aware date representation.    * not implemented
		#    %X : language-aware time representation.    * not implemented
		y => sprintf("%02d", substr($year, -2, 2)),
		Y => sprintf("%04d", $year),
		Z => $tz
	);
	my $key = join('|', keys %conv);
	$form_str =~ s/\%($key)/$conv{$1}||$1/ges;

	return $form_str;
}


#-----------------------------------------------------------------------------
# 
#-----------------------------------------------------------------------------


#-----------------------------------------------------------------------------
# File
#-----------------------------------------------------------------------------

# &print_file($file, \$str, $mode);
sub print_file
{
	my ($file, $str_ref, $mode) = @_;
	$mode ||= '>';

	my $str = $$str_ref;
	open(OUT, "$mode$file") 
		|| croak "Content-Type: text/plain\n\nCan't open $file\n";
	flock(OUT, 2);
	print OUT $str;
	flock(OUT, 8);
	close(OUT);
}

# $str = &read_file($file)
sub read_file
{
	my $file = shift;
	
	open(IN, $file) || croak "Content-Type: text/plain\n\nCan't open $file\n";
	my $str = "";
	while(my $s = <IN>){
		$str .= $s;
	}

	return $str;
}

#-----------------------------------------------------------------------------
# Encode, Decode
#-----------------------------------------------------------------------------


# $result = &html_encode($str)
sub html_encode
{
	my ($t) = @_;
	my %cv = ('&' => '&amp;', '"' => '&quot;',
		'<' => '&lt;',  '>' => '&gt;');
	$t =~ s/([&<>"])/$cv{$1}||$1/ge;
	$t;
}

# $result = &html_encode2($str)
sub html_encode2
{
	my ($t) = @_;
	return html_encode(html_decode($t));
}

# $result = &html_encode_minimal($str)
sub html_encode_minimal
{
	my ($t) = @_;
	my %cv = ('"' => '&quot;', '<' => '&lt;',  '>' => '&gt;');
	$t =~ s/([<>"])/$cv{$1}||$1/ge;
	$t;
}

# $result = &html_decode($str)
sub html_decode 
{
	my ($t) = @_;
	my %cv = ('&amp;' => '&', '&quot;'=> '"',
		'&lt;' => '<', '&gt;' => '>');
	my $s = join('|', keys %cv);
	$t =~ s/($s)/$cv{$1}||$1/ge;
	$t;
}

sub die_handler {
	my $msg = shift;
	$msg = html_encode($msg);

	print "Content-Type: text/html; charset=UTF-8\n\n";
	print qq(<html>
	<head>
		<title>Error Occured</title>
		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	</head>
	<body bgcolor="#eeeeee" text="#333333">
		<h2>Error Occured</h2><br>
		$msg
	</body>
</html>
);

	exit(1);
}


# $str = &url_encode($str)
sub url_encode
{
	my $arg = shift;
	$arg =~ s/(\W)/'%'.unpack("H2", $1)/ego;
	$arg =~ tr/ /+/; 

	return $arg;
}

# $str = &url_decode($str)
sub url_decode
{
	my $arg = shift;
	$arg =~ tr/+/ /;
	$arg =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/ego;

	return $arg;
}

# character code conversion
#
# convert_char_code(\$str, 'UTF-8');
my $default_encoding = 'UTF-8';

sub convert_char_code
{
	my $str_ref = shift;
	my $enc_to = shift || $default_encoding;

	my %encode_list = ('utf-8'=>'utf8', "euc-jp"=>'euc-jp', 
		'shift_jis'=>'sjis');
	$enc_to = $encode_list{lc($enc_to)}||$enc_to;

	if($$str_ref)  {
		if(!$ENCODE_MODULE) {
		# use Jcode.pm
			Jcode::convert($str_ref, $enc_to);	# UTF-8 に変換

			return 1;
		}
		else {
		# use Encode.pm
			Encode::Alias::define_alias( 'sjis' => 'shiftjis' );
			my @suspects_list = qw(euc-jp shiftjis 7bit-jis);
			my $decoder = Encode::Guess::guess_encoding($$str_ref, @suspects_list);
			
			if(ref $decoder){
			# successful guess
				from_to($$str_ref, $decoder->name, $enc_to);
				return 1;
			}else{
			# guess failed
				if($$str_ref =~ /encoding=["'](.+?)["']/s) {
				# XML Declaration
					my $from_name = '';
					if($from_name = $encode_list{lc($1)}||$1){
						eval{from_to($$str_ref, $from_name, $enc_to);};
						if($@) {return 0;}
						else   {return 1;}
					}
				}
			}
		}
	}

	return 0;
}


# RNA::Utils::execute_sub_routine(\&sub, $is_background);
sub execute_sub_routine
{
	my ($sub_routine, $is_background) = @_;
	
	if($is_background) {
	# バックグラウンド処理
		my $pid = '';
		$|=1; # 即時出力
		if($pid = fork) {
			# 親プロセス
			return 1;
		}
		elsif (defined $pid) {
			# 子プロセス
			close(STDOUT);
			&$sub_routine;
			exit;
		}
		else {
		# エラー
			croak("Can't generate process.");
		}
	}
	else {
	# 通常の処理
		&$sub_routine;
		return 1;
	}
}



1;
__END__

