#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# XML-RPC 
#
#  !! Do not  'use' or 'require' the modules that access STDIN such as 'CGI.pm'
#	【注意】標準入力にアクセスするような処理を行わないこと
#
#  http://www.xmlrpc.com/weblogsCom	: Weblogs.Com XML-RPC interface
#  http://www.xmlrpc.com/  : XML-RPC Home Page
#  http://search.cpan.org/author/KULCHENKO/SOAP-Lite-0.55/lib/XMLRPC/Lite.pm
#    XMLRPC::Lite
#
#==============================================================================


package RNA::XMLRPC;
use strict;

use vars qw($VERSION);
use XMLRPC::Transport::HTTP;
use RNA::BaseAPI;

$VERSION = '0.1';

my @dispatch_list = ('weblogUpdates', 'rna');

# &RNA::XMLRPC::set_config_file($filename);
sub set_config_file
{
	RNA::BaseAPI::config_file($_[0]);
}


# $handle = RNA::XMLRPC::server_handle;
sub server_handle
{
	# XML::Parser を使わず，XML::Parser::Lite を使う
	$XMLRPC::Constants::DO_NOT_USE_XML_PARSER = 1;
	
	# サーバーに，パッケージを dispatch 
	return XMLRPC::Transport::HTTP::CGI->dispatch_to(@dispatch_list)->handle;
}

sub load_module
{
	return RNA::BaseAPI::load_module;
}

# &RNA::XMLRPC::_fault;
sub _fault
{
	my $info = RNA::BaseAPI::get_fault_info;
	die SOAP::Fault->faultcode($info->{code})->faultstring($info->{string});
}

#=============================================================================
package weblogUpdates;
use strict;

# $hash_ref = &ping($blog_name, $blog_uri)
#
# MT からの自動 ping の受付
#
# Usage :
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: RNA
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 206
#
#<?xml version="1.0"?><methodCall><methodName>weblogUpdates.ping</methodName><params><param><value>Scripting News</value></param><param><value>http://www.scripting.com/</value></param></params></methodCall>
#
sub ping
{
	shift if UNIVERSAL::isa($_[0] => __PACKAGE__);
	my ($blog_name, $blog_uri) = @_;

	my $rna = &RNA::XMLRPC::load_module;
	$rna->update;

	return {
		flerror => XMLRPC::Data->type('boolean'=>0), 
		message => 'Thanks for the ping.'};
}


#=============================================================================
package rna;
use strict;


#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: unknown (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 102
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.getVersion</methodName></methodCall>
sub getVersion
{
	shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	my $version = &RNA::BaseAPI::getVersion;
	if($version) {
	# success
		return $version;
	}
	else {
		return &RNA::XMLRPC::_fault;
	}
}


#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: Radio UserLand/7.1b7 (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 822
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.addClip</methodName><params><param><value><struct><member><name>username</name><value>anonymous</value></member><member><name>passwd</name><value></value></member><member><name>title</name><value>XML-RPC Title</value></member><member><name>link</name><value>XML-RPC-Link</value></member><member><name>description</name><value>XML-RPC Description</value></member><member><name>subject</name><value>XML-RPC DC Category</value></member><member><name>creator</name><value>XML-RPC DC creator</value></member><member><name>date</name><value>XML-RPC DC Date</value></member><member><name>source</name><value>XML-RPC AG source</value></member><member><name>sourceURL</name><value>XML-RPC AG SourceURL</value></member></struct></value></param></params></methodCall>
#
sub addClip
{
	shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	if(RNA::BaseAPI::addClip(@_)) {
	# success
		return XMLRPC::Data->type('boolean'=>1);
	}
	else {
	# Error
		return &RNA::XMLRPC::_fault;
	}
}


#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: unknown (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 99
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.getDate</methodName></methodCall>

sub getDate
{shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	my $rna = &RNA::XMLRPC::load_module;
	my $config = $rna->{Config};
	my $recent_site_list = $config->{RecentSiteList}->{RSS};
	
	my $rss = new RNA::RSS;
	$rss->parsefile($recent_site_list);

	return $rss->{channel}->{dc}->{date};
}

#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: unknown (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 100
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.getSites</methodName></methodCall>
sub getSites
{shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	my $list_ref 
		= &RNA::BaseAPI::getSitesInCategory($RNA::BaseAPI::ALL_CATEGORY);
	if($list_ref) {
	# success
		return $list_ref;
	}
	else {
		return &RNA::XMLRPC::_fault;
	}
}


#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: unknown (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 178
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.getSitesInCategory</methodName><params><param><value><string>news</string></value></param></params></methodCall>
sub getSitesInCategory
{
	shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	my $category = shift;
	my $list_ref = &RNA::BaseAPI::getSitesInCategory($category);
	if($list_ref) {
	# success
		return $list_ref;
	}
	else {
		return &RNA::XMLRPC::_fault;
	}
}


#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: Radio UserLand/7.1b7 (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 607
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.addSite</methodName><params><param><value><struct><member><name>username</name><value>anonymous</value></member><member><name>passwd</name><value></value></member><member><name>title</name><value>CNET Japan</value></member><member><name>link</name><value>http://japan.cnet.com/rss/index.rdf</value></member><member><name>description</name><value>XML-RPC Description</value></member><member><name>subject</name><value>news</value></member><member><name>creator</name><value>CNET</value></member></struct></value></param></params></methodCall>
#
sub addSite
{
	shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	if(RNA::BaseAPI::addSite(@_)) {
	# success
		return XMLRPC::Data->type('boolean'=>1);
	}
	else {
		return &RNA::XMLRPC::_fault;
	}
}


#[Request]
#POST /~msano/rna/rna-xmlrpc.cgi HTTP/1.0
#User-Agent: unknown (WinNT)
#Host: www-kasm.nii.ac.jp
#Content-Type: text/xml
#Content-length: 109
#
#<?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>rna.getSiteCategories</methodName></methodCall>
sub getSiteCategories
{
	shift if UNIVERSAL::isa($_[0] => __PACKAGE__);

	my $list_ref = &RNA::BaseAPI::getSiteCategories;
	if($list_ref) {
	# success
		return $list_ref;
	}
	else {
		return &RNA::XMLRPC::_fault;
	}
}



#=============================================================================
1;

