#!/usr/bin/perl
#=============================================================================
# RNA Configuration 
#=============================================================================
#
# 設定ファイルの編集など，各種設定を行うスクリプト
#
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

# ライブラリ
use strict;
use lib qw(lib extlib);
use CGI;
# use XML::RSS;
use RNA::RSS;
use RNA;
use RNA::OPML;
use RNA::Utils qw( :all );
use RNA::RSS::WebEdit;
use RNA::Auth;

my $cgi = new CGI;
$cgi->param;
&die_handler($cgi->cgi_error) if ($cgi->cgi_error);

#----------------------------------
# 基本設定
#----------------------------------
my $self_uri = $cgi->url(-relative=>1);

#==============================================================================
# main
#==============================================================================

my $rna = RNA->new(Config => './config.pl');
my $config = $rna->{Config};
my $list_rss_file = $config->{SiteList}->{File};
my $hatena_flag = $config->{HatenaFlag};


#----------------------------------
# 認証
#----------------------------------
if($config->{Authentication}->{ConfigureCGI}) {
	unless($rna->authenticate(cgi=>$cgi)){
		print $cgi->redirect($config->{AuthenticationCGIURI} 
					. '?transfer=' . url_encode($self_uri));
		exit;
	}
}


if($cgi->param('target') eq 'update_site_list'){

	my %hash = (
		action => $cgi->param('action'),

		select => $cgi->param('select'),
		link    => $cgi->param('link'),
		title  => $cgi->param('title'),
		description => $cgi->param('description'),
		creator => $cgi->param('creator'),
		subject => $cgi->param('subject')
	);
	
	foreach my $key ('title', 'description', 'creator', 'subject'){
		unless($hash{$key}){
			$hash{$key} = $config->empty_value;
		}
	}

	&update_site_list(%hash);
}
elsif($cgi->param('target') eq 'update_hatena'){
# はてなアンテナ
	my %hash = (
		action => $cgi->param('action'),

		link    => $cgi->param('link'),
		subject => $cgi->param('subject')
	);
	&update_hatena(%hash);
}
elsif($cgi->param('target') eq 'opml') {
# OPML
	my $fh = $cgi->upload('file') || '';

	my %hash = (
		action => $cgi->param('action'),

		fh    => $fh
	);
	&opml(%hash);
}
elsif($cgi->param('target') eq 'update_password'){
# username, passwd
	my %hash = (
		action => $cgi->param('action'),

		username => $cgi->param('username'),
		passwd   => $cgi->param('passwd')
	);

	&update_password(%hash);
}
else{
	&show_config_page;
}


#=============================================================================
# Sub routine
#=============================================================================

#
# &update_site_list(%hash);
sub update_site_list
{
	my %arg = @_;

	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($list_rss_file);

	my $err_msg = &RNA::RSS::WebEdit::update_rss($list_rss, \%arg, 'back');
	if($err_msg) {
		&die_handler($err_msg);
	}

	foreach my $item (@{$list_rss->{items}}) {
		delete $item->{ag};
	}

	#-------------------------------------
	# 出力
	#-------------------------------------
	&set_site_list($list_rss_file, $list_rss);
	
	#-----------------------------------
	# self_url へのリダイレクション
	#-----------------------------------
	print $cgi->redirect($self_uri);
}

#--------------------------------
# はてなアンテナ
#--------------------------------
# &update_hatena(%hash);
sub update_hatena
{
	my %arg = @_;
	my $new_link = $arg{link};
	my $subject = $config->{HatenaFlag};

	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($list_rss_file);
	my @items = @{$list_rss->{items}};

	# hatena の <item> の取り出し
	my $hatena_item = {};
	foreach my $i (0 .. $#items) {
		if($items[$i]->{dc}->{subject} eq $hatena_flag) {
			$hatena_item = splice(@items, $i, 1);
			last;
		}
	}
	$list_rss->{items} = \@items;

	# 更新
	$hatena_item->{title} = $config->empty_value;
	$hatena_item->{link} = $new_link;
	$hatena_item->{dc}->{subject} = $subject;
	if($hatena_item->{link}) {
		$list_rss->add_item(%{$hatena_item});
	}

	#-------------------------------------
	# 出力
	#-------------------------------------
	&set_site_list($list_rss_file, $list_rss);
	
	#-----------------------------------
	# self_url へのリダイレクション
	#-----------------------------------
	print $cgi->redirect($self_uri);
}


# &set_site_list($file, $rss)
sub set_site_list
{
	my ($file, $rss) = @_;

	&set_rss($rss, 
		$config->{SiteList}->{Title}, $config->{SiteList}->{URI}, 
		$config->{SiteList}->{Description}, $rna->{Date}, $config->{Owner}, 
		$config->{Language});
	my $str = $rss->as_string;
	&print_file($file, \$str);
}

#--------------------------------
# OPML
#--------------------------------
# &opml(%hash);
sub opml
{
	my %arg = @_;
	my $fh = $arg{fh};

	if ($arg{action} =~ /import/i){
	# Import OPML site list

		if($fh) {
			my $str = '';
			my $buffer = '';
			my $buf_size = 256;
			
			while (read($fh, $buffer, $buf_size)) {
				$str .= $buffer;
			}

			&import_opml($str, $list_rss_file);
		}
	}
	else{
		&die_handler('不正なリクエスト（OPML Import）-> ' . $arg{action});
	}

	#-----------------------------------
	# self_url へのリダイレクション
	#-----------------------------------
	print $cgi->redirect($self_uri);
}

# &import_opml($opml_str, $rss_file)
sub import_opml
{
	my $str = shift;
	my $output_file = shift;

	convert_char_code(\$str);  # UTF-8
#	print "Content-type: text/plain; charset=UTF-8\n\n$str"; && exit;

	my $opml = new RNA::OPML(encoding=>'UTF-8');
	eval{ $opml->parse($str); };
	if($@) {
		&die_handler("Invalid OPML. ($@)")
	}
	
	my $rss = new RNA::RSS;
	$rss = $opml->to_rss($rss);
# print "Content-type: text/plain\n\n" . $opml->as_string;
# print "Content-type: text/plain\n\n" . $rss->as_string; exit;

	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($output_file);
	my @uri_list = map {$_->{link}} @{$list_rss->{items}};
	my %exist_uri_hash = ();
	foreach my $uri (@uri_list) {
		$exist_uri_hash{$uri} = 1;
	}

	# $list_rss と重複しないものだけ，$rss から取得
	foreach my $item (@{$rss->{items}}) {
		unless($exist_uri_hash{$item->{link}}) {
			$list_rss->add_item(%{$item});
		}
	}

#print "Content-type: text/plain\n\n" . $list_rss->as_string;


	#-------------------------------------
	# 出力
	#-------------------------------------
	&set_site_list($output_file, $list_rss);
}


#-----------------------------------
# Authentication
#-----------------------------------
#	&update_password(%hash);
sub update_password
{
	my %arg = @_;
	my $username = $arg{username};
	my $passwd = $arg{passwd};
	my $passwd_file = $config->{Authentication}->{PasswordFile};
	my $auth = new RNA::Auth(file=>$passwd_file);

	if ($arg{action} =~ /add/i){
		$auth->add_user($username, $passwd);
	}
	elsif ($arg{action} =~ /update/i) {
		$auth->update_user($username, $passwd);
	}
	elsif ($arg{action} =~ /delete/i) {
		$auth->delete_user($username);
	}
	else  {
		&die_handler('Invalid Request (Authentication) -> ' . $arg{action});
	}

	#-----------------------------------
	# Redirection to self_url 
	#-----------------------------------
	print $cgi->redirect($self_uri);
}


#-----------------------------------
# configure ページ
#-----------------------------------
#	&show_config_page;
sub show_config_page
{
	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($list_rss_file);
	my @items = @{$list_rss->{items}};

	# duplicated URI
	my @dup_list = &get_duplicated_uri((map {$_->{link}} @items));
	my $dup_check = join('|', @dup_list);

	#------------------------------------
	# はてなアンテナの取り出し
	#------------------------------------
	my $hatena_item = {};
	foreach my $i (0 .. $#items) {
		if($items[$i]->{dc}->{subject} eq $hatena_flag) {
			$hatena_item = splice(@items, $i, 1);
			last;
		}
	}

	# subject のリスト
	my %subject_hash = get_subject_list(@items);
	my $subject_select_options = '';
	foreach my $s (sort keys %subject_hash) {
		my $n = $subject_hash{$s};
		$subject_select_options .= "<option value=\"" . encode($s) . "\">" 
			. encode($s) . " ($n)</option>\n";
	}

	# HTTP header
	my $http_header = 
		$cgi->header(-type=>'text/html', 
			-charset => $config->{Encoding});

	my $indent = "\t\t";
	my $update_site_list_switch = join("\n$indent", 
		map {
			"case \"" . escape_uri($_->{link}) . "\" : \n" 
				. "$indent\tform.title.value = \"". encode($_->{title}) . "\"\;\n"
				. "$indent\tform.link.value = \"". escape_uri($_->{link}) . "\"\;\n"
				. "$indent\tform.description.value = \"" 
					. encode($_->{description}) . "\"\;\n"
				. "$indent\tform.creator.value = \""
					. encode($_->{dc}->{creator}) . "\"\;\n"
				. "$indent\tform.subject.value = \"". 
					encode($_->{dc}->{subject}) . "\"\;\n"
#				. "$indent\tform.date.value = \"". encode($_->{dc}->{date}) . "\"\;\n"
#				. "$indent\tform.source.value = \"". encode($_->{ag}->{source}) . "\"\;\n"
#				. "$indent\tform.sourceURL.value = \"". escape_uri($_->{ag}->{sourceURL}) . "\"\;\n"
				. "$indent\t" 
					.($dup_check && $_->{link}=~/$dup_check/?"alert('Duplicated entry! Modify $list_rss_file directly');":'')
					. "\n"
				. "$indent\tbreak\;"
		} @items
	);

	$indent = "\t\t";
	my $update_site_list_options = "\n";
	foreach my $i (0 .. $#items) {
		my $item = $items[$i];
		
		$update_site_list_options .= "$indent" 
			. sprintf("<option value=\"%s\">%04d. %s \&nbsp\;\&nbsp\; : %s</option>\n", escape_uri($item->{link}), $i+1, escape_uri($item->{link}), encode($item->{title}));
	}
	
	#-------------------------
	# テンプレート
	#-------------------------
	my $template = &read_file($config->{Template}->{Configure});
	my %text_hash = %{$config->{Template}->{Element}};
	$text_hash{__RNAUpdateSiteListSwitch__} = $update_site_list_switch;
	$text_hash{RNASiteList}  = $list_rss_file;
	$text_hash{__RNAUpdateSiteListOptions__}  = $update_site_list_options;
	$text_hash{__RNASubjectSelectOptions__} = $subject_select_options;

	# はてなアンテナ
	$text_hash{__RNAHatenaAntennaURI__} = $hatena_item->{link};
	$text_hash{__RNAHatenaAntennaSubject__} = $hatena_item->{dc}->{subject};	
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	# 出力
	print $http_header;
	print $template;
}



# @duplicated_uri_list = &get_duplicated_uri(@uri_list)
sub get_duplicated_uri
{
	return RNA::RSS::WebEdit::get_duplicated_uri(@_);
}

# 	my %subject_hash = get_subject_list(@items);
#
# %subject_hash = ('sub1' => 4)
sub get_subject_list
{
	#return RNA::RSS::WebEdit::get_subject_list(@_);
	my @items = @_;
	
	my %hash = ();
	foreach my $item (@items) {
		my $subject = $item->{dc}->{subject};
		if(!$config->is_empty_content($subject)){
			$hash{$subject} += 1;
		}
	}
	return %hash;
}

# $str = escape_uri($str)
sub escape_uri { return RNA::RSS::WebEdit::escape_uri(@_);}

# $str = encode($str)
sub encode {return RNA::RSS::WebEdit::encode(@_);}
